% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/npc.R
\name{npc}
\alias{npc}
\title{Calculate the Neyman-Pearson Classifier from a sample of class 0 and class 1.}
\usage{
npc(x = NULL, y, method = c("logistic", "penlog", "svm", "randomforest",
  "lda", "nb", "ada", "custom"), kernel = "radial", score = NULL,
  pred.score = NULL, alpha = 0.05, delta = 0.05, split = 1,
  split.ratio = 0.5, n01.min = 10, adaptive = FALSE, n.cores = 1,
  randSeed = 0)
}
\arguments{
\item{x}{n * p observation matrix. n observations, p covariates.}

\item{y}{n 0/1 observatons.}

\item{method}{classification method.
\itemize{
\item logistic: Logistic regression. \link{glm} function with family = 'binomial'
\item penlog: Penalized logistic regression with LASSO penalty. \code{\link[glmnet]{glmnet}} in \code{glmnet} package
\item svm: Support Vector Machines. \code{\link[e1071]{svm}} in \code{e1071} package
\item randomforest: Random Forest. \code{\link[randomForest]{randomForest}} in \code{randomForest} package
\item Linear Discriminant Analysis. lda: \code{\link[MASS]{lda}} in \code{MASS} package
\item nb: Naive Bayes. \code{\link[e1071]{naiveBayes}} in \code{e1071} package
\item ada: Ada-Boost. \code{\link[ada]{ada}} in \code{ada} package
\item custom: a custom classifier. score vector needed.
}}

\item{kernel}{kernel used in the svm method. Default = 'radial'.}

\item{score}{score vector corresponding to y. Required when method  = 'custom'.}

\item{pred.score}{predicted score vector for the test sample. Optional when method  = 'custom'.}

\item{alpha}{the desirable control on type I error. Default = 0.05.}

\item{delta}{the violation rate of the type I error. Default = 0.05.}

\item{split}{the number of splits for the class 0 sample. Default = 1. For ensemble
version, choose split > 1.  When method = 'custom',  split = 0 always.}

\item{split.ratio}{the ratio of splits used for the class 0 sample to train the
classifier. Default = 0.5.}

\item{n01.min}{the minimum number of class 0 observations used for training classifier. Default = 10.}

\item{adaptive}{whether to adaptively choose the number of observations used for
determining the cutoff when 1/2 n0 < lower bound < n0. Default = F.}

\item{n.cores}{number of cores used for parallel computing. Default = 1. WARNING:
windows machine is not supported.}

\item{randSeed}{the random seed used in the algorithm.}
}
\value{
An object with S3 class npc.
\item{fit}{the fit from the specified classifier.}
\item{score}{the score vector for each observation.}
 \item{cutoff}{thecutoff determined via bootstrap to achieve the specified type I error
  control.}
 \item{sign}{whether class 1 has a larger average score than the
  class 0.}
 \item{method}{the classification method.}
 \item{loc.prob}{the
  percentile used to determine the cutoff for the specified type I error
  control.}
  \item{split}{the number of splits used.}
  \item{n0toosmall}{whether the class 0 sample size is too small for the desirable type
  I error control}
}
\description{
Give a type I error upper bound alpha and the violation upper bound delta, \code{npc} calculate the Neyman-Pearson Classifier
which control the type I error with in alpha with probability at least 1-delta
}
\examples{
set.seed(0)
n = 1000
x = matrix(rnorm(n*2),n,2)
c = 1+3*x[,1]
y = rbinom(n,1,1/(1+exp(-c)))
xtest = matrix(rnorm(n*2),n,2)
ctest = 1+3*xtest[,1]
ytest = rbinom(n,1,1/(1+exp(-ctest)))

##Use svm classifier and the default type I error control with alpha=0.05
fit = npc(x, y, method = 'svm')
pred = predict(fit,xtest)
fit.score = predict(fit,x)
accuracy = mean(pred$pred.label==ytest)
cat('Overall Accuracy: ',  accuracy,'\\n')
ind0 = which(ytest==0)
typeI = mean(pred$pred.label[ind0]!=ytest[ind0]) #type I error on test set
cat('Type I error: ', typeI, '\\n')

##Ensembled svm classifier with split = 11,  alpha=0.05
fit = npc(x, y, method = 'svm', split = 11)
pred = predict(fit,xtest)
accuracy = mean(pred$pred.label==ytest)
cat('Overall Accuracy: ',  accuracy,'\\n')
ind0 = which(ytest==0)
typeI = mean(pred$pred.label[ind0]!=ytest[ind0]) #type I error on test set
cat('Type I error: ', typeI, '\\n')

##Now, change the method to logistic regression and change alpha to 0.1
fit = npc(x, y, method = 'logistic', alpha = 0.1)
pred = predict(fit,xtest)
accuracy = mean(pred$pred.label==ytest)
cat('Overall Accuracy: ',  accuracy,'\\n')
ind0 = which(ytest==0)
typeI = mean(pred$pred.label[ind0]!=ytest[ind0]) #type I error on test set
cat('Type I error: ', typeI, '\\n')

##Now, change the method to adaboost
fit = npc(x, y, method = 'ada', alpha = 0.1)
pred = predict(fit,xtest)
accuracy = mean(pred$pred.label==ytest)
cat('Overall Accuracy: ',  accuracy,'\\n')
ind0 = which(ytest==0)
typeI = mean(pred$pred.label[ind0]!=ytest[ind0]) #type I error on test set
cat('Type I error: ', typeI, '\\n')

##A 'custom' npc classifier with y and score.
fit2 = npc(y = y, score = fit.score$pred.score,
pred.score = pred$pred.score, method = 'custom')
}
\references{
Xin Tong, Yang Feng, and Jingyi Jessica Li (2016), Neyman-Pearson (NP) classification algorithms and NP receiver operating characteristic (NP-ROC) curves, manuscript, http://arxiv.org/abs/1608.03109.
}
\seealso{
\code{\link{nproc}} and \code{\link{predict.npc}}
}

